@@ -11,7 +11,7 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * RCS: @(#) $Id: tclCompCmds.c,v 1.33 2002/08/26 17:38:54 msofer Exp $
+ * RCS: @(#) $Id: tclCompCmds.c,v 1.39 2003/02/07 01:07:05 mdejong Exp $
  */
 
 #include "tclInt.h"
@@ -1558,10 +1558,6 @@ TclCompileIncrCmd(interp, parsePtr, envPtr)
 	    code = TclCompileTokens(interp, incrTokenPtr+1, 
 	            incrTokenPtr->numComponents, envPtr);
 	    if (code != TCL_OK) {
-		if (code == TCL_ERROR) {
-		    Tcl_AddObjErrorInfo(interp,
-	                    "\n    (increment expression)", -1);
-		}
 		goto done;
 	    }
 	}
@@ -1649,7 +1645,7 @@ TclCompileLappendCmd(interp, parsePtr, envPtr)
     CompileEnv *envPtr;		/* Holds resulting instructions. */
 {
     Tcl_Token *varTokenPtr, *valueTokenPtr;
-    int numValues, simpleVarName, isScalar, localIndex, numWords;
+    int simpleVarName, isScalar, localIndex, numWords;
     int code = TCL_OK;
 
     /*
@@ -1672,7 +1668,6 @@ TclCompileLappendCmd(interp, parsePtr, envPtr)
 	 */
         return TCL_OUT_LINE_COMPILE;
     }
-    numValues = (numWords - 2);
 
     /*
      * Decide if we can use a frame slot for the var/array name or if we
@@ -1708,16 +1703,6 @@ TclCompileLappendCmd(interp, parsePtr, envPtr)
 		goto done;
 	    }
 	}
-#if 0
-    } else {
-	/*
-	 * We need to carefully handle the two arg case, as lappend
-	 * always creates the variable.
-	 */
-
-	TclEmitPush(TclRegisterNewLiteral(envPtr, "", 0), envPtr);
-	numValues = 1;
-#endif
     }
 
     /*
@@ -2195,9 +2180,8 @@ TclCompileRegexpCmd(interp, parsePtr, envPtr)
 {
     Tcl_Token *varTokenPtr;	/* Pointer to the Tcl_Token representing
 				 * the parse of the RE or string */
-    int i, len, code, exactMatch, nocase;
-    Tcl_Obj *patternObj;
-    CONST char *str;
+    int i, len, code, nocase, anchorLeft, anchorRight, start;
+    char *str;
 
     /*
      * We are only interested in compiling simple regexp cases.
@@ -2223,7 +2207,7 @@ TclCompileRegexpCmd(interp, parsePtr, envPtr)
 	    /* Not a simple string - punt to runtime. */
 	    return TCL_OUT_LINE_COMPILE;
 	}
-	str = varTokenPtr[1].start;
+	str = (char *) varTokenPtr[1].start;
 	len = varTokenPtr[1].size;
 	if ((len == 2) && (str[0] == '-') && (str[1] == '-')) {
 	    i++;
@@ -2247,7 +2231,7 @@ TclCompileRegexpCmd(interp, parsePtr, envPtr)
      * If it has a '-', it could be an incorrectly formed regexp command.
      */
     varTokenPtr = varTokenPtr + (varTokenPtr->numComponents + 1);
-    str = varTokenPtr[1].start;
+    str = (char *) varTokenPtr[1].start;
     len = varTokenPtr[1].size;
     if ((varTokenPtr->type != TCL_TOKEN_SIMPLE_WORD) || (*str == '-')) {
 	return TCL_OUT_LINE_COMPILE;
@@ -2261,64 +2245,82 @@ TclCompileRegexpCmd(interp, parsePtr, envPtr)
 	return TCL_OK;
     }
 
+    /*
+     * Make a copy of the string that is null-terminated for checks which
+     * require such.
+     */
+    str = (char *) ckalloc((unsigned) len + 1);
+    strncpy(str, varTokenPtr[1].start, (size_t) len);
+    str[len] = '\0';
+    start = 0;
+
+    /*
+     * Check for anchored REs (ie ^foo$), so we can use string equal if
+     * possible. Do not alter the start of str so we can free it correctly.
+     */
+    if (str[0] == '^') {
+	start++;
+	anchorLeft = 1;
+    } else {
+	anchorLeft = 0;
+    }
+    if ((str[len-1] == '$') && ((len == 1) || (str[len-2] != '\\'))) {
+	anchorRight = 1;
+	str[--len] = '\0';
+    } else {
+	anchorRight = 0;
+    }
+
     /*
      * On the first (pattern) arg, check to see if any RE special characters
      * are in the word.  If not, this is the same as 'string equal'.
-     * We can use strchr here because the glob chars are all in the ascii-7
-     * range.  If -nocase was specified, we can't do this because INST_STR_EQ
-     * has no support for nocase.
      */
-    
-    if (Tcl_RegExpCompile(NULL, str) == NULL) {
-	/*
-	 * This is a bad RE.  Let it complain at runtime.
-	 */
-	return TCL_OUT_LINE_COMPILE;	
+    if ((len > (1+start)) && (str[start] == '.') && (str[start+1] == '*')) {
+	start += 2;
+	anchorLeft = 0;
     }
-#if 0
-    if ((len > 2) && (*str == '.') && (str[1] == '*')) {
-	str += 2; len -= 2;
-    }
-    if ((len > 2) && (str[len-3] != '\\')
+    if ((len > (2+start)) && (str[len-3] != '\\')
 	    && (str[len-2] == '.') && (str[len-1] == '*')) {
 	len -= 2;
+	str[len] = '\0';
+	anchorRight = 0;
     }
-#endif
-    if ((len > 1) && (str[0] == '^') && (str[len-1] == '$')
-	    && (str[len-2] != '\\')) {
-	/*
-	 * It appears and exact search was requested (ie ^foo$), so strip
-	 * off the special chars and signal exactMatch.
-	 */
-	str++; len -= 2;
-	exactMatch = 1;
-    } else {
-	exactMatch = 0;
-    }
-    
-    patternObj = Tcl_NewStringObj(str, len);
-    Tcl_IncrRefCount(patternObj);
-    code = (strpbrk(Tcl_GetString(patternObj), "*+?{}()[].\\|^$") != NULL);
-    Tcl_DecrRefCount(patternObj);
-    if (code) {
-	/* We don't do anything with REs with special chars yet. */
+
+    /*
+     * Don't do anything with REs with other special chars.  Also check if
+     * this is a bad RE (do this at the end because it can be expensive).
+     * If so, let it complain at runtime.
+     */
+    if ((strpbrk(str + start, "*+?{}()[].\\|^$") != NULL)
+	    || (Tcl_RegExpCompile(NULL, str) == NULL)) {
+	ckfree((char *) str);
 	return TCL_OUT_LINE_COMPILE;
     }
-    if (exactMatch) {
-	TclEmitPush(TclRegisterNewLiteral(envPtr, str, len), envPtr);
+
+    if (anchorLeft && anchorRight) {
+	TclEmitPush(TclRegisterNewLiteral(envPtr, str+start, len-start),
+		envPtr);
     } else {
 	/*
 	 * This needs to find the substring anywhere in the string, so
-	 * use string match and *foo*.
+	 * use string match and *foo*, with appropriate anchoring.
 	 */
 	char *newStr  = ckalloc((unsigned) len + 3);
-	newStr[0]     = '*';
-	strncpy(newStr + 1, str, (size_t) len);
-	newStr[len+1] = '*';
-	newStr[len+2] = '\0';
-	TclEmitPush(TclRegisterNewLiteral(envPtr, newStr, len+2), envPtr);
+	len -= start;
+	if (anchorLeft) {
+	    strncpy(newStr, str + start, (size_t) len);
+	} else {
+	    newStr[0] = '*';
+	    strncpy(newStr + 1, str + start, (size_t) len++);
+	}
+	if (!anchorRight) {
+	    newStr[len++] = '*';
+	}
+	newStr[len] = '\0';
+	TclEmitPush(TclRegisterNewLiteral(envPtr, newStr, len), envPtr);
 	ckfree((char *) newStr);
     }
+    ckfree((char *) str);
 
     /*
      * Push the string arg
@@ -2335,7 +2337,7 @@ TclCompileRegexpCmd(interp, parsePtr, envPtr)
 	}
     }
 
-    if (exactMatch && !nocase) {
+    if (anchorLeft && anchorRight && !nocase) {
 	TclEmitOpcode(INST_STR_EQ, envPtr);
     } else {
 	TclEmitInstInt1(INST_STR_MATCH, nocase, envPtr);
@@ -2376,6 +2378,7 @@ TclCompileReturnCmd(interp, parsePtr, envPtr)
 {
     Tcl_Token *varTokenPtr;
     int code;
+    int index = envPtr->exceptArrayNext - 1;
 
     /*
      * If we're not in a procedure, don't compile.
@@ -2385,6 +2388,22 @@ TclCompileReturnCmd(interp, parsePtr, envPtr)
 	return TCL_OUT_LINE_COMPILE;
     }
 
+    /*
+     * Look back through the ExceptionRanges of the current CompileEnv,
+     * from exceptArrayPtr[(exceptArrayNext - 1)] down to 
+     * exceptArrayPtr[0] to see if any of them is an enclosing [catch].
+     * If there's an enclosing [catch], don't compile.
+     */
+
+    while (index >= 0) {
+	ExceptionRange range = envPtr->exceptArrayPtr[index];
+	if ((range.type == CATCH_EXCEPTION_RANGE) 
+		&& (range.catchOffset == -1)) {
+	    return TCL_OUT_LINE_COMPILE;
+	}
+	index--;
+    }
+
     switch (parsePtr->numWords) {
 	case 1: {
 	    /*
